;
; clrchn.asm - CLRCHN, CLALL kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Clall & Clrchn
;
; Used registers: A X (Y preserved, A=0!!!)
; Status: not useful
; Status byte response:
; ST=$00 if no error
; ST=$03 timeout
; ST=$80 if device not present
;**************************************************

newclall	.proc
		ldx #MAX_OPEN_FILES
		#lda0
qho		sta open_files_logical_exp-1,x;Kernal does this, however
		dex			;closing files would be better
		bne qho
		ldx #MAX_BUFFERS
qho2		sta buffer_logical_exp-1,x;Kernal does this, however
		dex			;closing files would be better
		bne qho2

newclrchn	.proc
		.if SUPERCPU=0
		tsx
                inx
                lda $100,x
                cmp #$ce
                bne xx
                inx
                lda $100,x		;Yet another nice JiffyDOS hack
                cmp #$f2		;to make CONTROL+D work ;)
                bne xx
                #psr newclose+1
                php
                lda #$6f
                pha
                #psr nmi_return
                .fi

xx		#ldatax buffer_output_exp
		sta buffer_currentp_exp	;OP_do_command_channel
		lda buffer_info_exp,x
		cmp #INFO_STATUS
		bne clrchn_restore.clrchn_ano1
                jsr chkoutclrchncmdchannel
		#jmps clrchn_restore.clrchn_ano2

clrchn_restore	.proc
		#psr switch_16k
		#ldx_l_a buffer_output_exp
		lda buffer_info_exp,x

clrchn_ano1	cpx #STDOUT
		beq clrchn_ano2
		cmp #INFO_KERNAL
		bne clrchn_ano2
		lda #3
		cmp buffer_drive_exp,x
		bcs clrchn_ano2

		lda #STDOUT
		sta buffer_output_exp	;to fix unlisten loop
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		lda #<(buffer_info_exp+STDOUT)
		sta newchrout_ram_at_exp
		.fi

                .if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                lda dos_accel_internal_exp
                cmp #1
                .fi

		jsr OP_cfg16up
                .if JIFFYDOS_ACCEL_CHRINOUT=1
		jsr iecunlisten
                .else
                
                .if ((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)
                bcs nx
                jsr $edfe
                jmp nx2
nx              jsr iecunlisten
nx2
                .else
                jsr $edfe
                .fi
                
                .fi		;JIFFYDOS ACCEL CHRINOUT

		jsr OP_cfg16down

clrchn_ano2	lda #STDOUT
		sta buffer_output_exp
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		lda #<(buffer_info_exp+STDOUT)
		sta newchrout_ram_at_exp
		.fi

		#stz_a ActualInput
		lda #3
		sta ActualOutput

		#ldx_l_a buffer_input_exp
		cpx #STDIN
		beq clrchn_ano3
		lda buffer_info_exp,x
		cmp #INFO_KERNAL
		bne clrchn_ano3
		lda #3
		cmp buffer_drive_exp,x
		bcs clrchn_ano3x

		lda #STDIN
		sta buffer_input_exp	;to fix untalk loop
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		lda #<(buffer_info_exp+STDIN)
		sta newchrin_ram_at_exp
		sta newgetin_ram_at_exp
		.fi

                .if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                lda dos_accel_internal_exp
                cmp #1
                .fi

		jsr OP_cfg16up
                .if JIFFYDOS_ACCEL_CHRINOUT=1
		jsr iecuntalk
                .else
                
                .if ((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)
                bcs nx3
                jsr $edef
                jmp nx4
nx3             jsr iecuntalk
nx4
                .else
                jsr $edef
                .fi
                
                .fi		;JIFFYDOS ACCEL CHRINOUT

		jsr OP_cfg16down
clrchn_ano3	clc
clrchn_ano3x
		lda #STDIN
		sta buffer_input_exp
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		lda #<(buffer_info_exp+STDIN)
		sta newchrin_ram_at_exp
		sta newgetin_ram_at_exp
		.fi

		#lda0			;Lame kernal (see $f6f4...)
		jmp exit2basic
		.pend

chkoutclrchncmdchannel .proc
                #phy_a
		jsr OP_save_zeropage_scpu01up
		jsr OP_XIT_do_command_channel;IDE64 command_channel?
                #OP_scpu01down
		#ply_a
		jmp OP_restore_zeropage
                .pend
