;
; open.asm - OPEN kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Open
;
; Used registers: A X Y
; Status: C
; C=1:
; A=1 too many files
; A=2 file open
; A=4 file not found (tape)
; A=5 device not present, and filename is given
; A=6 not input file
; A=7 (not output file)
; A=9 illegal device number (tape)
; Status byte response:
; Cleared!
; ST=$80 if device not present, and filename is given
;
; Notes:
; If logical number=0 then err(not input file)
; If logical number exists then err(file open)
; If open_files>9 then err(too many files)
; Secondary address 128..255 then no communication to the serial bus
;**************************************************

;--------------------------------------
; OPEN
;--------------------------------------
newopen		.proc
                .if DEBUGIO=1
                lda debug_io_exp
                beq kd
                #lda0
                sta debug_io_exp
                #psr $ffc3
                php
                pha
                #phx_a
                #phy_a
                lda #3
                jmp monitor
kd		lda #1
		sta debug_io_exp
                .fi

		ldx ActualLogical
		beq not_input_file	;Logical file number 0 is invalid

		jsr search_logical_X_st0;ST=$00, open_files_currentp_exp
		beq file_alr_open

		#lda0
		jsr search_logical_A	;search empty entry
		bne too_many_files

		lda ActualSecAddr
                and #$8f
		ora #$60
		sta ActualSecAddr
		sta open_files_secondary_exp,x
		lda ActualDev
		sta open_files_device_exp,x
		#devicehandler hard_open,exit_illegal_device,pclink_open2,kernal_open

too_many_files	lda #KERNAL_TOO_MANY_FILES
                .byte SKIP2
not_input_file	lda #KERNAL_NOT_INPUT_FILE
                .byte SKIP2
file_alr_open	lda #KERNAL_FILE_OPEN
                #jmps exit_kernal_error
		.pend

kernal_open	.proc
		tya
		beq vd			;keyboard
		cmp #31
		bcs open_ignore		;serial drive illegal above 30
		cmp #3
		beq vd			;screen
		bcs serial

		#psr $f384		;Go to CBM kernal tape or rs232
                gcc vd

serial		ldy ActualSecAddr
		bmi vd			;secondary addr bit 7 set
		ldy ActualFNLength
		beq vd			;no filename

		pha
		#OP_scpu01up
		jsr OP_save_zeropage_copy_filename_to_path

		jsr OP_cfg16up
		pla
		jsr ieclisten		;listen
		lda ActualSecAddr
                ora #$f0		;open
		jsr iecsecond		;second
                bit Status
		bpl oe
np              jsr OP_cfg16down
                jsr OP_restore_zeropage_scpu01down
                jmp exit_device_not_present

oe      	ldx #0
cy		#sta_cfg_open
		lda path_exp,x
		#sta_cfg_16
		beq oy
		jsr iecout		;ciout
		inx
		gne cy
oy
		jsr iecunlisten		;unlsn
                jsr OP_cfg16down
                jsr OP_restore_zeropage_scpu01down

vd		lda #INFO_KERNAL	;Must be a standard CBM device...
		.byte SKIP2

open_ignore	.proc
		lda #INFO_IGNORE

open_ok_mode	.proc
		#ldx_l_ open_files_currentp_exp
		sta open_files_info_exp,x
		lda ActualLogical
openback	.proc
		sta open_files_logical_exp,x;Now it's officially open
		lda ActualDev
openclcexit    	.proc
		clc
		jmp exit2basic
                .pend

open_buffer_ok_mode .proc
		#ldx_l_ buffer_currentp_exp
		sta buffer_info_exp,x
		lda ActualLogical
		sta buffer_logical_exp,x;Now it's officially allocated
		#ldx_l_ open_files_currentp_exp
		#jmps openback
		.pend
;----------------------------
;Open from IDE64
;
hard_open	.proc
		#lda0
		jsr search_buffer_A
		beq buffavail		
                sec
                jsr XIT_open            ;All buffers taken: 70, no channel
                gcs open_ignore

buffavail	lda current_drive_exp
		sta buffer_drive_exp,x

		lda ActualSecAddr
                bmi open_ignore		;secondary addr bit 7 set
		and #$0f
		eor #$0f
		bne open_std_file
;----------------------------
;Open error channel from IDE64
;
		sta buffer_address_low_exp,x
                lda #$ff
		sta buffer_address_end_low_exp,x
                jsr OP_buffer_start
		sta buffer_address_end_hi_exp,x
                ora #$80
                sta buffer_address_hi_exp,x
                lda #BUFFERMODE_ERROR
                sta buffer_type_exp,x

		lda ActualFNLength
		beq hard_open_nerror

		#psr hard_open_cha
		#psr switch_16k

hard_open_nerror
		lda #INFO_STATUS
		gne open_buffer_ok_mode
;----------------------------
;Open file from IDE64
;
open_std_file	lda ActualFNLength
		beq open_ignore		;no filename...

		#OP_scpu01up
                jsr OP_save_zeropage_copy_filename_to_path
                clc
		jsr XIT_open
		pha
		jsr OP_restore_zeropage_scpu01down
		pla
		bcc open_buffer_ok_mode
		gcs open_ignore
;----------------------------
;Execute command channel stuff
;
hard_open_cha	#sta_cfg_open
		pha
                #OP_scpu01up
                jsr OP_save_zeropage_copy_filename_to_path
                #ldx_l_a buffer_currentp_exp
		jsr OP_XIT_execute_15command_open
		jsr OP_restore_zeropage_scpu01down
                pla
                #jmps openclcexit
;----------------------------------------------------------
;Open from harddisk
;
XIT_open	#xit_init

		bcs noc

		lda ActualSecAddr
		and #$0f
                pha
                lsr
		beq spc 		;0, 1 special can open multiple

		ldx #MAX_OPEN_FILES
lp2		dex
		bmi spc
		lda open_files_logical_exp,x;file open?
		beq lp2
		lda ActualDev
		cmp open_files_device_exp,x;on this drive?
		bne lp2
		lda ActualSecAddr
		cmp open_files_secondary_exp,x;channel correct?
		bne lp2
inva            sec
noc		gcs LWOP_reporterror_70_xit;channel already open
spc             

		jsr OP_get_current_drive_type
		and #$c0
		cmp #$80		;cdrom?
		php

		jsr OP_init_buffer_pointers
		#lda0
		sta buffer_address_end_hi_exp,x
		sta buffer_address_end_low_exp,x;Auto read first sector
		lda #>512
		plp
		bne *+4
		lda #>2048
		clc
		adc buffer_start,x
		sta buffer_address_max_hi_exp,x;Size of buffer
		
		lda path_exp
		cmp #"$"
		bne open_ndir
;---------------------------------------
;Directory open
;
		pla
		beq ms			;0 is normal directory
		lsr
inva2           beq inva                ;1 is invalid
		lda #1			;2-14 is raw
ms		pha

		jsr XIT_init_drive	;-=> error

		pla
		ora #2*2
		jsr XIT_dir_first	;-=> error

		jsr next_sector_to_direntry_next_sector_exp

		lda #>b_dir_searchstring_exp;backup searchstring
		jsr buffer_subpart_cp
                lda partition_currentp_exp
                sta buffer_partition_exp,x

hthon		#lda_l_y_x path_exp
		#sta_zp_y current_path
		beq mfw
		iny
		gne hthon
mfw
		lda #>b_dir_type_format_exp;backup type
		jsr buffer_subpart_cp

hthon2		#lda_l_y_x path_searchtype_exp,x
		#sta_zp_y current_path
		beq mfw2
		iny
		gne hthon2
mfw2
		lda #>b_dir_misc_exp
		jsr buffer_subpart_cp

		ldx #direntry_backup_end_exp-direntry_backup_start_exp-1
nvw		lda direntry_backup_start_exp,x
		#sta_zp_y current_path
		iny
		dex
		bpl nvw

		lda #BUFFERMODE_DIR
                gne openmoderead	;-=> ok
;---------------------------------------
open_ndir      	cmp #"#"
		bne open_ndirect
;---------------------------------------
;Direct channel open
;
		pla
		lsr
		beq inva2		;channels 2-14 are valid only

		jsr LWOP_reporterror_ok
		jsr OP_init_buffer_pointers
                sta buffer_address_end_low_exp,x
		lda buffer_address_max_hi_exp,x
                #sbc1c
		sta buffer_address_end_hi_exp,x

		lda #BUFFERMODE_DIRECT
		ldy #INFO_BLK_RW_DIRTY
		gne openmode   		;-=> ok
;---------------------------------------
;Normal file open
;
open_ndirect    pla

		jsr XIT_init_drive

		lda #RESOLV_OPENFIRST_MODE
		jsr OP_XIT_resolv_path_part;-=> error
		php

		ldy #3
ds		lda path_searchtype_exp
		cmp invl1,y
		bne vr
		lda path_searchtype_exp+1
		cmp invl2,y
		bne vr
		lda path_searchtype_exp+2
		cmp invl3,y
		bne vr
		gcs LWOP_reporterror_64_xit;illegal filetype -> file type mismatch

vr		dey
		bpl ds

		lda resolv_anything_exp
		sec
		beq LWOP_reporterror_62_xit

		#ldx_l_a buffer_currentp_exp
		#lda0
		sta buffer_filepos2_exp,x
		sta buffer_filepos3_exp,x
		sta buffer_filepos4_exp,x
                lda partition_currentp_exp
                sta buffer_partition_exp,x

		lda resolv_open_mode_exp
		cmp #"r"
		bne open_file_nread

open_file_read	pla			;pop

		jsr direntrytofsizecopy

		lda direntry_mode_exp
                .if DIRMODE_CFS
                cmp #DIRMODE_CFS
                .fi
                beq open_file_read_cfs
                cmp #DIRMODE_JOLIET
                beq open_file_read_iso
                cmp #DIRMODE_ISO
                beq open_file_read_iso
                cmp #DIRMODE_OFS
                beq open_file_read_ofs
                cmp #DIRMODE_FAT2
                beq open_file_read_fat
                sec
                gcs LWOP_reporterror_62_xit;unsupported filesystem

open_file_read_cfs

		jsr cfsopen

		jsr save_file_state

                lda #BUFFERMODE_FILE
                gne openmoderead

open_file_read_iso
		txa
                asl
                asl
                tax
                ldy #3
ro	      	lda next_sector,y
		sta buffer_isofile_start_exp,x
                inx
                dey
                bpl ro

		lda #BUFFERMODE_ISOFILE
                gne openmoderead
                
open_file_read_ofs

		lda buffer_start,x
                jsr init_pom1_y

ek		ldx oldfsbyteorder,y
                lda next_sector,x
                #sta_zp_y pom1
                iny
                cpy #4
                bcc ek

		lda #BUFFERMODE_OFSFILE
openmoderead	ldy #INFO_BLK_READ
openmode  	#ldx_l_ buffer_currentp_exp
		sta buffer_type_exp,x
		tya
                clc
	     	rts			;-=> ok

open_file_read_fat

		lda #>b_fat_current_exp
		jsr buffer_subpart_cp	;restore searchstring
                
                ldy #4
                #lda0
                #sta_zp_y current_path
                dey
ir              lda next_sector,y
                #sta_zp_y current_path
                dey
                bpl ir
                
 		lda #BUFFERMODE_FAT
                gne openmoderead

open_file_nread	cmp #"w"
		beq open_file_wr
                plp
		cmp #"m"
                clc
		beq mod
		cmp #"a"
		sec
		bne LWOP_reporterror_64_xit;not implemented yet

mod		php
		
                jsr direntrytofsizecopy

		jsr cfsopen

		jsr OP_buffer_start
		ora #>b_cfs_file_bitmap_exp
                jsr find_free_start

                ldx #3
iri             lda direntry_f_size_exp,x
                sta path_exp+2,x
                lda resolv_this_sector_exp,x
		sta cfs_file_dirsect_exp,x
                dex
                bpl iri
                
                plp
                bcc napp

                jsr save_file_state

		#ldx_l_a buffer_currentp_exp
                lda #INFO_BLK_WRITE
                jsr seekappend
		#jmps kk

napp		jsr kk
		lda #INFO_BLK_RW
                rts

open_file_wr   	plp
		bcs open_file_write	;not found
		sec
                lda path_rewrite_exp
		beq LWOP_reporterror_63_xit;file exists

                #phx_a			;it's not the most optimal way for "@:", but it works

		#cnext resolv_this_sector_exp , next_sector

		lda #ram_exp_dir	;read in dir sector
		jsr XIT_read_a_sector	;(RESOLV_CREAT only reads in dir if there's an empty entry!)

		#cnext direntry_f_pointer_exp , next_sector

                jsr rewrite_file	;remove file

                ldx #15
np              lda direntry_f_name_exp,x
                sta path_exp,x
                dex
                bpl np
                #stx_l_a resolv_empty_entry_found_exp
                ldx #4
np2             lda direntry_f_filetype_exp-1,x
                sta path_searchtype_exp-1,x	;1 byte too much, but that's no problem
                lda resolv_this_sector_exp-1,x
                sta resolv_empty_entry_exp-1,x
                dex
                bne np2
                #stx_l_a path_exp+16
                #plx_a

open_file_write	
                #lda0
                jsr direntrytofsize	;clear
                sta buffer_treeposlo_exp,x ;set the current treepos to 0
                sta buffer_treeposhi_exp,x
                lda #$80		;dirty
                sta buffer_treestate_exp,x

                lda setup_atribut2_exp
                asl

		lda buffer_start,x
		ora #>b_cfs_file_bitmap_exp
                ldx #0
                bcs *+4
                ldx #"/"
		jsr XIT_prealloc_dir_sector

		lda #FIND_FREE_METADATA
		jsr find_free_space	;allocate tree sector

		jsr buffer_tree_pom1	;filepos is 0 anyway

		ldx #4
co              lda next_sector-1,x
                sta cfs_file_tree_exp-1,x     	;save tree sector
                sta cfs_file_treeroot_exp-1,x	;backup treeroot
                .if SUPERCPU=1
                stz szam-1,x
                .else
                lda #0
                sta szam-1,x            ;filelen=0
                .fi
                dex
                bne co                  
                .if SUPERCPU=1
                txa
                .fi
                tay

clearme		#sta_zp_y pom1          ;zero out tree sector
		iny
		bne clearme
		inc pom2
clearme2       	#sta_zp_y pom1
		iny
		bne clearme2

		lda #FTYPE_NORMAL | FATTR_EXEC | FATTR_READ | FATTR_WRITE | FATTR_DELETE
                sec
		jsr XIT_finalize_direntry	;-=> error

		#cnext next_sector , cfs_file_dirsect_exp

kk              lda resolv_entry_offs_exp
		sta cfs_file_diroffs_exp

		jsr save_file_state

		lda #BUFFERMODE_FILE
		ldy #INFO_BLK_WRITE
                gne openmode            ;-=> ok
                
cfsopen		#lda0			;empty
                sta buffer_treestate_exp,x

		#cnext next_sector , cfs_file_treeroot_exp;backup treeroot
                rts

invl1		.text "drdl"		;"DRCDL"
invl2		.text "eein"		;"EEBIN"
invl3		.text "llrk"		;"LLMRK"
		.pend
                
oldfsbyteorder	.byte 1,2,0,3

next_sector_to_direntry_next_sector_exp .proc
		#cnext next_sector , direntry_next_sector_exp
                rts
                .pend

;---------------------------------------
direntrytofsizecopy .proc
		lda #$ff
direntrytofsize	.proc
		tay
		and direntry_f_size_exp
		sta buffer_filesize1_exp,x
                tya
		and direntry_f_size_exp+1
		sta buffer_filesize2_exp,x
                tya
		and direntry_f_size_exp+2
		sta buffer_filesize3_exp,x
                tya
		and direntry_f_size_exp+3
		sta buffer_filesize4_exp,x
		rts
                .pend
;---------------------------------------
OP_init_buffer_pointers .proc
		jsr OP_buffer_start
                #sbc1c
		sta buffer_address_hi_exp,x
		lda #255
		sta buffer_address_low_exp,x
		rts		;A=255 on exit!
		.pend
;----------------------------------
search_logical_X_st0 .proc
		#stz_a Status
		txa
search_logical_X_close .proc
                bne search_logical_A
                .byte SKIP2		;0 is invalid
;----------------------------
search_logical_A .proc
		ldx #MAX_OPEN_FILES
lp		dex
		bmi exit
		cmp open_files_logical_exp,x
		bne lp
		#stx_l open_files_currentp_exp
exit		rts
		.pend
;-------------------------
search_logical_X_chkinout .proc
		jsr search_logical_X_st0
		beq search_buffer_A_readfiletables
                pla
                pla
                lda #KERNAL_FILE_NOT_OPEN
                #jmps exit_kernal_error

search_buffer_A_readfiletables .proc
		pha
		lda open_files_logical_exp,x
		sta ActualLogical
		lda open_files_device_exp,x
		sta ActualDev
		lda open_files_secondary_exp,x
		sta ActualSecAddr
		pla
;-------------------------
search_buffer_A	.proc
		ldx #MAX_BUFFERS
lp		dex
		bmi exit
		cmp buffer_logical_exp,x
		bne lp
		#stx_l buffer_currentp_exp
exit		rts
		.pend
                
;-------------------------
pclink_open2	.proc
		#lda0
		jsr search_buffer_A
		bne open_ignore2       	;All buffers taken???

		jsr OP_init_buffer_pointers
		#lda0
		sta buffer_address_end_hi_exp,x
		sta buffer_address_end_low_exp,x;Auto read first sector
		lda #>2048
		clc
		adc buffer_start,x
		sta buffer_address_max_hi_exp,x;Size of buffer
                lda #BUFFERMODE_PCLINK
                sta buffer_type_exp,x

		lda ActualSecAddr
                sta buffer_pclink_secaddr,x
		bmi open_ignore2

		jsr OP_save_zeropage_copy_filename_to_path
		jsr pclink_open
                pha
                jsr OP_restore_zeropage
                pla
		bcc open_buffer_ok_mode
                lda #$80
                sta Status
                #psr $f707		;device not present

open_ignore2	jmp open_ignore
		.pend
