;
; setup.asm - Setup screen
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
;Setup
;
;**************************************************

tmp		= $f7
togg		= $f9
temptmp		= $99

ingetin		= $f142

call_setup	.proc
		sei
		lda #6
		sta msgmode
		bit $d011
		bpl *-3
		bit $d011
		bmi *-3
		sta $d020
		sta $d021
		lda #11
		sta $d011
		lda 650
		sta ism
		lda 648
		sta m648
		lda #4
		sta 648
		jsr LWNO_print_imediate
		.byte 13,5,147,8,14,154	;White, clr, sh+C=, lower, lt.blue

		.text "           "
;		.if PC_DUART=1
;		.text "  "
;		.fi

		.text "IDE64 IDEDOS "
		#version
		.if SUPERCPU=1
		.text "S"
		.fi
;		.text " "

;		.if PC_DUART=0
;		.text 128+"P",128+"A",128+"R",128+"A",128+"L",128+"L",128+"E",128+"L"
;		.else
;		.text 128+"D",128+"U",128+"A",128+"R",128+"T"
;		.fi

		.byte 13
		.text "           CMOS SETUP UTILITY",13
		.text "            THE IDE64 PROJECT",13

		.byte 5,176,17,157,157,174,221;white, +, down, left, left, |
		.byte 0

		lda #<$4c7
		ldy #>$4c7
                jsr settogg

rc		lda #93
		sta (togg),y
		iny
		sta (togg),y

		tya
		clc
		adc #39
		tay
		bcc *+4
		inc togg+1
		eor #168
		bne rc

		tay
		ldx #21
		jsr $fff0
		jsr LWNO_print_imediate
		.byte 171,13,157,179,221,155
		.text "Run/Stop : Quit          CRSR : Move  "
		.byte 5,221,221,155
		.text "C=       : Save          +/-  : Modify"
		.byte 5,221,173,0

		lda #<tt0
		ldy #>tt0
		jsr printpg

		sei
		lda #<irq
		sta $0314
		lda #>irq
		sta $0315
		ldx #37
rt		lda #64
		sta $0479,x
		sta $0749,x
		sta $07c1,x
		dex
		bpl rt
		lda #125		
		sta $07e7
		lda #3
		sta retry		;ram teszt retry
		sta $dd00
		lsr			;#1
                sta hdsettings
		sta $cc			;cursor off
		ror			;#128
		sta 650			;ismetles
		asl			;#0
		ldx #63
rh		sta $033f,x
		dex
		bne rh
		stx hol
		stx $d012
		stx $dc0e
		stx $d01c
		lda #7
		sta $d017
		sta $d01b
		sta $d01d
		sta $d015
		dex
		stx $0340
		stx $0343
		stx $0346
		stx $0349
                ldy #2
ir		lda #13
		sta $07f8,y
		lda #10
		sta $d027,y
                dey
                bpl ir
		stx showclock
		bit $dc0d

nf		#sta_cfg_open
		jsr LWOP_get_cmos_setup
		bcs joez

		lsr hdsettings
		jsr write_ram_burst

		dec retry
		bne nf
		geq k5			

joez		lda #%10010001
                jsr cmos_start
                jsr cmos_read_byte
                sta setup_accu_old
                ldx #6
ore             cmp accuconst,x
                beq lv
                dex
                bne ore
lv              stx setup_accu
		jsr cmos_end
                
                lda #%10000001
                jsr cmos_start
                jsr cmos_read_byte
                jsr cmos_end
                tax
                bpl nstopped
                and #$7f		;start the clock if stopped
                tax
                lda #%10000000
                jsr updatec2
nstopped
		lda setup_accu_old
                and #$f0                
                cmp #$a0
                beq kfd
                ldx #31
vd              lda battlow,x
                sta $4a4,x
                lda #3
                sta $d8a4,x
                dex
                bpl vd

kfd		jsr highlight

		lda #27
		sta $d011

kl2re		cli

kl		jsr check_move
		cmp #13			;return
		bne k3
		lda #>kl2re
		pha
		lda #<kl2re
execmpont	pha
		.if SUPERCPU=1
                lda setup_atribut_exp
                jmp (ampont,x)
		.else
		lda ampont+1,x
		pha
		lda ampont,x
		pha
                lda setup_atribut_exp
		rts
                .fi
k3
		ldx 654			;C=
		cpx #2
		beq saveandexit2

		cmp #3
		beq k5
		gne kl

saveandexit
		pla
		pla
saveandexit2
		sei
		lda $d011
		bpl *-3
		jsr write_ram_burst
                ldy setup_accu
                ldx accuconst,y
                cpx setup_accu_old
                beq nset
                lda #%10010000
                jsr updatec2
nset		jsr OP_hdinit_drivenumbers
		lda hdsettings
                bne a2
                lda #DRIVES-1
                sta tmp

mvx             #ldatax tmp
                sta current_drive_exp
		lda drive_type_exp,x
		beq a1
		jsr set_power_management_and_features
a1		dec tmp
                bpl mvx
a2
k5		#stz_a $d01a
		#stz_aa $d015
		#stz_aa $cf			;hide cursor
		jsr LWOP_get_cmos_setup
		#sta_cfg_16
		bcc iujq
		lda retry
		bne rvg
iujq		jsr $e544
		clc
		ldx #12
                ldy #5
                jsr $fff0
		jsr LWNO_print_imediate
		.text "CMOS RAM error, press any key."
		.byte 0

		lda $d011
		bpl *-3
		lda $d011
		bmi *-3
		ldx #2
		stx $d020
		stx $d021
		lda #27
		sta $d011
		dex
		stx $d01a
		jsr ingetin
		beq *-3
		#stz_a $d01a
rvg		lda ism
		sta 650
		lda m648
		sta 648
		jmp $e544

discardexit
		pla
		pla
		#jmps k5

;---------------------------------------
;---------------------------------------
standardstart
		jsr backlight2
		lda #<tt1
		ldy #>tt1
		jsr printpg
		lda #mpont1-mpont
                jsr highlight2
		#stz_a showclock
		#sta_cfg_open

standardloop
		lda #<$52b
		ldy #>$52b
		jsr settogg
		lda #%11000000
		and setup_atribut_exp
		cmp #%01000000
		bne *+3
		tya
                asl
                rol
                rol
		jsr printany
                ldy #0
		lda setup_boot_drive_exp
                clc
		jsr pn2

	       	lda #<$57b
	       	ldy #>$57b
	      	jsr settogg
		lda #%00100000
iid             pha
                .if (DE32_EXISTS=0) | (SUPERCPU=1)
		and #%11111110
                .fi
		jsr toggle
                pla
                lsr
                bne iid
		lda direct_write_exp
		and #1
		jsr toggle4
                lda setup_accu
                clc
                adc #35
                jsr printany
		lda #%10000000
		jsr toggle5
		lda #%01000000
		jsr toggle5

stdx		jsr check_move
		bne sk3
		lda #>(standardloop-1)
		pha
		lda #<(standardloop-1)
		#jmps execmpont
sk3
		jsr checkcommstop
		bne stdx

		#lda0


returnmain      pha
		jsr backlight2
		lda #<tt0
		ldy #>tt0
		sty showclock
		jsr printpg
		pla
highlight2    	sta hol
		jsr highlight
		lda #7
		sta $d015
		rts

check_move	#sta_cfg_16
		ldx #1
		stx $d01a
		jsr ingetin
		dec $d01a
		#sta_cfg_open
		cmp #29			;right
		beq *+6
		cmp #17			;down
		bne sk1
		jsr backlight
		ldx hol
		cpx #mpont1-mpont-2
		beq sk1k
		cpx #mpont4-mpont-2
		beq sk1k
		cpx #mpont3-mpont-2
		beq sk1k
		cpx #mpont2-mpont-2
		beq sk1k
		cpx #mponte-mpont-2
		beq sk1k
		inx
		inx
sk1k		stx hol
		jsr highlight
		#jmps check_move
sk1
		cmp #157		;left
		beq *+6
		cmp #145		;up
		bne sk2
		jsr backlight
		ldx hol
		beq sk1k
		cpx #mpont1-mpont
		beq sk1k
		cpx #mpont4-mpont
		beq sk1k
		cpx #mpont3-mpont
		beq sk1k
		cpx #mpont2-mpont
		beq sk1k
		dex
		dex
		#jmps sk1k

sk2		ldx hol
		tay
		cmp #"-"
		beq *+8
		cmp #"+"
		beq *+4
		cmp #13
		rts
;---------------------------------------
;---------------------------------------
advancedstart   jsr backlight2
		lda #<tt2
		ldy #>tt2
		jsr printpg
		jsr LWNO_print_imediate
                .if DRIVES=4
		.text 155,13,13,29,"Second.",0
                .else
		.text 155,13,13,29,"Slave ",0
                .fi

		lda #<tt2i
		ldy #>tt2i
                sta tmp
                sty tmp+1
		jsr printpgin;O_print_ay
		lda #mpont2-mpont
		jsr highlight2
		#sta_cfg_open

advancedloop    lda #<$4db
		ldy #>$4db
		jsr settogg
		lda setup_powerman_exp
                jsr printpm1
		lda #%00001000
nn              pha
		jsr toggle3
                pla
                lsr
                bne nn
		lda setup_linear_r_hdd0_exp
		jsr printnum2_0
		lda setup_linear_w_hdd0_exp
		jsr printnum2_40

                .if DRIVES=4
		lda #<$4e5
		ldy #>$4e5
                .else
		lda #<$643
		ldy #>$643
                .fi
		jsr settogg
		lda setup_powerman_exp
                jsr printpm2
		lda #%10000000
nn2             pha
		jsr toggle3
                pla
                lsr
                cmp #%00001000
                bne nn2
		lda setup_linear_r_hdd1_exp
		jsr printnum2_0
		lda setup_linear_w_hdd1_exp
		jsr printnum2_40

                .if DRIVES=4
		lda #<$643
		ldy #>$643
		jsr settogg
		lda setup_powerman2_exp
                jsr printpm1
		lda #%00001000
nn3             pha
		jsr toggle6
                pla
                lsr
                bne nn3
		lda setup_linear_r_hdd2_exp
		jsr printnum2_0
		lda setup_linear_w_hdd2_exp
		jsr printnum2_40

		lda #<$64d
		ldy #>$64d
		jsr settogg
		lda setup_powerman2_exp
                jsr printpm2
		lda #%10000000
nn4             pha
		jsr toggle6
                pla
                lsr
                cmp #%00001000
                bne nn4
		lda setup_linear_r_hdd3_exp
		jsr printnum2_0
		lda setup_linear_w_hdd3_exp
		jsr printnum2_40
		.fi

advx		jsr check_move
		bne ak3
                lsr hdsettings
		#psr advancedloop
		lda ampont+1,x
		pha
		lda ampont,x
		pha
                #lda0
                tax
		rts

ak3             jsr checkcommstop
		bne advx
		lda #6
		gne returnmain

;---------------------------------------
;---------------------------------------
devicestart     jsr backlight2
		lda #<tt3
		ldy #>tt3
		jsr printpg

		lda #mpont3-mpont
		jsr highlight2
		#sta_cfg_open

deviceloop      lda #<$4b3
		ldy #>$4b3
		jsr settogg

		.comment
		.if DRIVES=4
		lda setup_drivenumbers_exp+4
		.else
		lda setup_drivenumbers_exp+2
                .fi
		jsr printnumdev3
                ldy #80
                .endc
		.if DRIVES=4
		lda setup_drivenumbers_exp+1
		.else
		lda setup_drivenumbers_exp+0
                .fi
		jsr printnumdev2
		.if DRIVES=4
		lda setup_drivenumbers_exp+2
		.else
		lda setup_drivenumbers_exp+1
                .fi
		ldy #40
		jsr printnumdev2
		.if DRIVES=4
		lda setup_drivenumbers_exp+3
		.else
		lda setup_drivenumbers_exp+2
                .fi
		ldy #80
		jsr printnumdev2
		.if DRIVES=4
		lda setup_drivenumbers_exp+4
		.else
		lda setup_drivenumbers_exp+3
                .fi
		ldy #120
		jsr printnumdev2

		lda #<($4b3+200)
		ldy #>($4b3+200)
		jsr settogg
		lda setup_drivenumbers_exp
		jsr printnumdev
		lda setup_drivenumbers_exp+1
		ldy #40
		jsr printnumdev
                .if DRIVES=4
		lda setup_drivenumbers_exp+2
		ldy #80
		jsr printnumdev
		lda setup_drivenumbers_exp+3
		ldy #120
		jsr printnumdev
                ldy #200
		lda setup_drivenumbers_exp+0
		jsr printnumdev2
                .else
                ldy #120
		lda setup_drivenumbers_exp+3
		jsr printnumdev
                .fi


devx		jsr check_move
		bne ak4
		#psr deviceloop
		lda ampont+1,x
		pha
		lda ampont,x
		pha
                ldx #0
		rts

ak4             jsr checkcommstop
		bne devx
		lda #4
		gne returnmain
;---------------------------------------
;---------------------------------------
colorstart      jsr backlight2
		lda #<tt4
		ldy #>tt4
		jsr printpg

		lda #mpont4-mpont
		jsr highlight2
		#sta_cfg_open

colorloop       lda #<$4fc
		ldy #>$4fc
                jsr settogg
                ldx #0
qq              #phx_a
		lda setup_mancolors_exp,x
                jsr printcol
                #plx_a
                inx
                cpx #7
                bcc qq
                ldx #0
qq2             #phx_a
		lda setup_mancolors_exp,x
                jsr printcol2
                #plx_a
                inx
                cpx #6
                bcc qq2
                
                lda #<$50e
		ldy #>$50e
                jsr settogg
		lda setup_paper_border_exp
		jsr printcol
		lda setup_paper_border_exp
		jsr printcol2
		lda setup_chars_exp
		jsr printcol

                lda #<$5fe
		ldy #>$5fe
                jsr settogg
		lda setup_moncolors_exp
		jsr printcol
		lda setup_moncolors_exp
		jsr printcol2

colx		jsr check_move
		bne ak5
		#psr colorloop
		lda ampont+1,x
		pha
		lda ampont,x
		pha
                ldx #0
		rts

ak5		jsr checkcommstop
		bne colx
		lda #2
		gne returnmain

restoredefault  lsr hdsettings
		#jmps cmos_default

;---------------------------------------
;---------------------------------------

settogg         sta togg
                sty togg+1
                ldy #0
                rts

checkcommstop  	ldx 654			;C=
		cpx #2
		beq *+4
		cmp #3
                rts
;---------------------------------------
irq		ldx showclock
		bmi noclock
		bne noclock2
		#sta_cfg_open
		jsr LWOP_read_clock
		jsr showtime
                lda #6
                sec
                sbc TV_machine_type_exp
                tax
		#sta_cfg_16
noclock2	dex
		stx showclock
noclock		jsr $ea87
		lda 651
		cmp #3
		lda #3
                sta $d019
		bcc mdf
		sta 651
mdf		jmp $ea81
;---------------------------------------
printnumdev3	and #15
		beq kkk
                clc
                adc #3
                gcc pn2

printnumdev2	jsr lsr4
printnumdev	and #15
		bne pn
kkk             ldx #"-"
                jsr p
                gne p

pn		clc
		adc #7
                gcc pn2
		
printnum2_40	ldy #40
		.byte SKIP2
printnum2_0	ldy #0
		sec
pn2		sty tmp
		sbc #99
		bcc qk
		ldx #49
		sbc #100
		bcc *+5
		inx
		sbc #100
		jsr p

qk		ldx #58
		dex
		adc #10
		bcc *-3
		cpx #49
		bcs qk3
		cpy tmp
		beq qk2
qk3		jsr p

qk2		adc #47
		tax
		jsr p
		ldx #32
		jsr p
p		pha
		txa
		sta (togg),y
		pla
		iny
		rts
;---------------------------------------
printpm2	jsr lsr4
printpm1       	eor #1
		and #15
		beq printany
                clc
                adc #3
                gcc printany

printcol2	jsr lsr4
printcol	and #15
		clc
                adc #19
printany       	tay
		ldx #255
rte		inx
		lda toggletexts,x
		bpl rte
		dey
		bpl rte
                asl

df2		inx
df		lsr
		iny
		sta (togg),y
                cpy #8
                bcs kr
		lda toggletexts,x
		asl
		bcc df2
                lda #64
                gne df

kr              lda togg
                adc #40-1
                sta togg
                bcc *+4
                inc togg+1
		rts
;---------------------------------------
		.if DRIVES=4
toggle6		and setup_features2_exp
		bne tin
                .fi
toggle5		and setup_atribut2_exp
		bne tin
toggle3		and setup_features_exp
		bne tin
toggle		and setup_atribut_exp
toggle4		beq printany
tin		lda #1
		gne printany
;--------------------------------------
tt0		.byte 17,17,158,12
		.text "STANDARD SETUP",13,13
		.byte 12
		.text "COLOR SETUP",13,13
		.byte 12
		.text "DEVICE NUMBERS",13,13
		.byte 12
		.text "ADVANCED SETUP",13,13
		.byte 12
		.text "RESTORE DEFAULTS",13,13
		.byte 12
		.text "SAVE & EXIT",13,13
		.byte 12
		.text "DISCARD & EXIT",5
		.byte 0
tt1		.text "Date (mm:dd:yy) :    ,"
		.byte 13
		.text "Time (hh:mm:ss) : ",29,29," : ",29,29," :",13
		.byte 13
                .if SUPERCPU=1
		.text "Start boot file :",13
                .else
		.text "Start boot file :",28,"CPULT:   ",13
                .fi
		.text "Boot device",16,":",13
		.text "Disk fastloader :",13
		.text "Set basic clock :",13
		.text "Keyboard repeat :",13
		.text "Lowercase chars :",13
		.text "Use dos wedge",16,":",13
		.text "C128 keyboard",16,":",13
		.text "Direct write",16,":",13
		.text "Accu charging",16,":",13
		.text "CMD emulation",16,":",13
		.text "Function keys",16,":"
		.byte 0
;---------------------------------------
tt3		;.text "LCD display     :",13
		;.byte 13
		.text "Serial device 8 :",13
		.text "Serial device 9 :",13
		.text "Serial device 10:",13
		.text "Serial device 11:",13
		.byte 13
                .if DRIVES=4
		.text "IDE 0 (p.master):",13
		.text "IDE 1 (p.slave) :",13
                .text "IDE 2 (s.master):",13
		.text "IDE 3 (s.slave) :",13
                .else
		.text "IDE 0 (master)",16,":",13
		.text "IDE 1 (slave)",16,":",13
                .fi
                .byte 13
		.text "PC-LINK",16,":"
		.byte 0
tt4		.text "Manager colors       System colors",13
		.byte 13
		.text "Paper",9,":",21,"Paper :",13
		.text "Border",9,":",21,"Border:",13
		.text "Panel",9,":",21,"Char  :",13
		.text "Act.panel",9,":",13
		.text "Dir",9,":",21,"Monitor colors",13
		.text "File",9,":",13
		.text "Sel.dir",9,":",21,"Paper :",13
		.text "Sel.file",9,":",21,"Char  :",13
		.text "Filebar",9,":",13
		.text "Misc",9,":",13
		.text "Window",9,":",13
		.text "Menubar",9,":",13
		.text "Input",9,":"
		.byte 0
;---------------------------------------
;Name of colors
		.enc screen
toggletexts   	.byte "D"+128,"isabled"
		.byte "E"+128,"nabled"
		.byte "P"+128,"ower on"
		.byte "A"+128,"lways"
                .byte "D"+128,"efault"
		.byte "2"+128," Min"
		.byte "1"+128," Min"
		.byte "4"+128," Min"
		.byte "3"+128," Min"
		.byte "6"+128," Min"
		.byte "5"+128," Min"
		.byte "8"+128," Min"
		.byte "7"+128," Min"
		.byte "1"+128,"5 Min"
		.byte "1"+128,"0 Min"
		.byte "3"+128,"0 Min"
		.byte "2"+128,"0 Min"
		.byte "2"+128," Hour"
		.byte "1"+128," Hour"
		.byte "B"+128,"lack"
		.byte "W"+128,"hite"
		.byte "R"+128,"ed"
		.byte "C"+128,"yan"
		.byte "P"+128,"urple"
		.byte "G"+128,"reen"
		.byte "B"+128,"lue"
		.byte "Y"+128,"ellow"
		.byte "O"+128,"range"
		.byte "B"+128,"rown"
	        .byte "L"+128,".red"
		.byte "D"+128,".gray"
		.byte "M"+128,"ed.gray"
		.byte "L"+128,".green"
		.byte "L"+128,".blue"
		.byte "L"+128,".gray"
		.byte "B"+128,"attery"
                .if OLDRESISTOR=1
		.byte "2"+128,"43 uA"		;6800 Ohm
		.byte "2"+128,"90 uA"
		.byte "3"+128,"33 uA"
		.byte "3"+128,"98 uA"
		.byte "4"+128,"09 uA"
		.byte "4"+128,"88 uA"		;Following byte must be >128! (128)
                .else
		.byte "0"+128,".4 mA"		;10 Ohm
		.byte "0"+128,".5 mA"
		.byte "0"+128,".8 mA"
		.byte "1"+128,".0 mA"
		.byte "1"+128,".8 mA"
		.byte "2"+128,".1 mA"		;Following byte must be >128! (128)
                .fi
                .enc none
;---------------------------------------
		.if DRIVES=4
tt2		.text 155,"Primary"
tt2i		.text " interface ",5,"Master    Slave",13
                .else
tt2		.text 155,"Master"
tt2i		.text " drive settings",5,13
		.fi
		.text "Power management:",13
		.text "Retry on error",16,":",13
		.text "Write cache",16,":",13
		.text "Read look-ahead :",13
		.text "Slow down CDROM :",13
		.text "Linear read  max:",13
		.text "Linear write max:"
                .byte 0
;---------------------------------------
prnthx		tax
		and #15
		ora #48
		sta $04bd,y
		txa
lsr4		lsr a
		lsr a
		lsr a
		lsr a
		rts

prnthx3		jsr prnthx
prnthx2		eor #48
		sta $04bc,y
		rts

showtime	
		.if SUPERCPU=0
		lda temptmp
		bit $01
		bmi ue
rr              ldy #128
                jsr prnthx
		bne *+4
		lda #16
		jsr prnthx2
                #ldatax $00
                ora #$80
                sta $00
                lda $01
                tay
                ora #$80
                sta $01
                stx $00
                sty $01
                #lda0

ue		clc
		sed
		adc #1
                cld
                sta temptmp
                bcs rr
                .fi

	      	ldy #41
		lda sec_exp		;sec
		jsr prnthx3

		ldy #36
		lda min_exp		;min
		jsr prnthx3

		ldy #31
		lda hour_exp		;hour
		jsr prnthx
		and #3
		bne *+4
		lda #16
		jsr prnthx2

		lda month_exp		;month
		and #%00011111
		cmp #16
		bcc *+4
		sbc #6
		tax
		lda dayc1+6,x
		sta $04b8
		lda dayc2+6,x
		sta $04b9
		lda dayc3+6,x
		sta $04ba

		ldy #0
		lda date_exp		;date
		jsr prnthx
		and #3
		bne *+4
		lda #16
		jsr prnthx2

		ldy #5
		lda year_exp		;year
		jsr prnthx3
		cmp #56			;19xx?

		ldx #49
		lda #57
		bcs *+5
		inx
		lda #48			;20xx
		sta $04c0
		stx $04bf

					;calculate day of week
		lda day_exp
                sta oldday
		lda year_exp		;year
		#andlsr $f0
		sta datepuff
		lsr a
		lsr a
		adc datepuff
		sta day_exp

		#ldatax date_exp       	;date
		#andlsr $f0
		sta datepuff
		lsr a
		lsr a
		adc datepuff
		sta datepuff
		txa
		and #15
		adc datepuff
		sta datepuff

		lda month_exp		;month
		and #%00011111
		cmp #16
		bcc *+5
		sbc #6
		clc
		tax

		lda year_exp		;year
		and #15
		adc day_exp
		sta day_exp
		lsr a
		bcs bw
		lsr a
		bcs bw2
		cpx #3

		.byte SKIP2
bw		lsr a
		sec
bw2		adc day_exp
		adc datepuff
		adc const1-1,x
		.if SUPERCPU=1
		xba
		lda year_exp
		asl
		xba
		.else
		ldx year_exp
		cpx #$80
		.fi
		sbc #7
		bcs *-2
		tax
		adc #8
		pha
		lda dayc1-249,x
		sta $04b3
		lda dayc2-249,x
		sta $04b4
		lda dayc3-249,x
		sta $04b5
                pla
                cmp oldday
                sta day_exp
                beq bx
		tax
		lda #$8a		;write day of week
                gne updatec2
bx              rts

const1		.byte 6,2,2,5,0,3,5,1,4,6,2,4
monl		.byte $31,$28,$31,$30,$31,$30,$31,$31,$30,$29,0,0,0,0,0,$31,$30,$31
fajt		.byte 14,11,16,2,3,4,8,7
;---------------------------------------
;Menu: x,y,length,routine
;---------------------------------------
L14		= $00
L11		= $20
L16		= $40
L2 		= $60
L3 		= $80
L4 		= $A0
L8 		= $C0
L7 		= $E0

mpont		.byte 13,6+L14
		.byte 13,8+L11
		.byte 13,10+L14
		.byte 13,12+L14
		.byte 13,14+L16
		.byte 13,16+L11
		.byte 13,18+L14

mpont1		.byte 24,4+L3   	;month
		.byte 28,4+L2   	;day
		.byte 31,4+L4   	;year
		.byte 19,5+L2   	;hour
		.byte 24,5+L2   	;min
		.byte 29,5+L2   	;sec
		.byte 19,7+L8
		.byte 19,8+L2
		.byte 19,9+L8
		.byte 19,10+L8
		.byte 19,11+L8
		.byte 19,12+L8
		.byte 19,13+L8
		.byte 19,14+L8
		.byte 19,15+L8
		.byte 19,16+L7
		.byte 19,17+L8
		.byte 19,18+L8
mpont4
		.byte 12,6+L8
		.byte 12,7+L8
		.byte 12,8+L8
		.byte 12,9+L8
		.byte 12,10+L8
		.byte 12,11+L8
		.byte 12,12+L8
		.byte 12,13+L8
		.byte 12,14+L8
		.byte 12,15+L8
		.byte 12,16+L8
		.byte 12,17+L8
		.byte 12,18+L8
		.byte 30,6+L8
		.byte 30,7+L8
		.byte 30,8+L8
		.byte 30,12+L8
		.byte 30,13+L8
mpont3
;		.byte 19,4+L2
		.byte 19,4+L2
		.byte 19,5+L2
		.byte 19,6+L2
		.byte 19,7+L2
		.byte 19,9+L2
		.byte 19,10+L2
                .if DRIVES=4
		.byte 19,11+L2
		.byte 19,12+L2
                .fi
		.byte 19,10+DRIVES+L2

mpont2		.byte 19,5+L8   	;powerm1
		.byte 19,6+L8   	;retry1
		.byte 19,7+L8   	;writecache1
		.byte 19,8+L8   	;lookahead1
		.byte 19,9+L8	  	;slowcd1
		.byte 19,10+L3  	;Linear
		.byte 19,11+L3  	;Linear
                .if DRIVES=4
                .byte 29,5+L8   	;powerm1
		.byte 29,6+L8   	;retry1
		.byte 29,7+L8   	;writecache1
		.byte 29,8+L8   	;lookahead1
		.byte 29,9+L8	  	;slowcd1
		.byte 29,10+L3  	;Linear
		.byte 29,11+L3  	;Linear
                .fi
		.byte 19,14+L8  	;powerm2
		.byte 19,15+L8  	;retry2
		.byte 19,16+L8  	;writecache2
		.byte 19,17+L8  	;lookahead2
		.byte 19,18+L8  	;slowcd2
		.byte 19,19+L3  	;Linear
		.byte 19,20+L3  	;Linear
                .if DRIVES=4
		.byte 29,14+L8  	;powerm2
		.byte 29,15+L8  	;retry2
		.byte 29,16+L8  	;writecache2
		.byte 29,17+L8  	;lookahead2
		.byte 29,18+L8  	;slowcd2
		.byte 29,19+L3  	;Linear
		.byte 29,20+L3  	;Linear
                .fi
mponte

ampont		#pword standardstart
		#pword colorstart
		#pword devicestart
		#pword advancedstart
		#pword restoredefault
		#pword saveandexit
		#pword discardexit

ampont1		#pword tmh
		#pword tdt
		#pword tyr
		#pword thr
		#pword tmn
		#pword tsec1
		#pword tboot
		#pword tbootd
		#pword tspeed
		#pword tbclk
		#pword trep
		#pword tlower
		#pword twedge
		#pword tc128key
		#pword tdirect
	       	#pword taccu
	       	#pword tcmd
	       	#pword tfnk
ampont4
		.word tman1-1
		.word tman2-1
		.word tman3-1
		.word tman4-1
		.word tman5-1
		.word tman6-1
		.word tman7-1
		.word tman8-1
		.word tman9-1
		.word tman10-1
		.word tman11-1
		.word tman12-1
		.word tman13-1
		.word tbg-1
		.word tbr-1
		.word tfg-1
		.word tmon1-1
		.word tmon2-1
ampont3
;		.word tlcd-1
		.word tdr8-1
		.word tdr9-1
		.word tdr10-1
		.word tdr11-1
		.word thd0-1
		.word thd1-1
                .if DRIVES=4
		.word thd2-1
		.word thd3-1
                .fi
		.word tpcl-1

ampont2		.word tpm1-1
		.word tretr0-1
		.word twch0-1
		.word tahed0-1
		.word tscd0-1
		.word tlin0-1
		.word twin0-1
		.word tpm2-1
		.word tretr1-1
		.word twch1-1
		.word tahed1-1
		.word tscd1-1
		.word tlin1-1
		.word twin1-1
                .if DRIVES=4
		.word tpm3-1
		.word tretr2-1
		.word twch2-1
		.word tahed2-1
		.word tscd2-1
		.word tlin2-1
		.word twin2-1
		.word tpm4-1
		.word tretr3-1
		.word twch3-1
		.word tahed3-1
		.word tscd3-1
		.word tlin3-1
		.word twin3-1
                .fi
amponte

;---------------------------------------
tfnk		lda #%01000000
		.byte SKIP2
tcmd		lda #%10000000
		eor setup_atribut2_exp
		sta setup_atribut2_exp
                rts

tspeed		eor #%00110000
tbclk		eor #%00011000
trep		eor #%00001100
tlower		eor #%00000110
		.if (DE32_EXISTS=1) & (SUPERCPU=0)
twedge		eor #%00000011
tc128key	eor #%00000001
		.else
twedge		eor #%00000010
tc128key	and #%11111110
		.fi
		sta setup_atribut_exp
		rts
tretr1		eor #%11000000
twch1		eor #%01100000
tahed1		eor #%00110000
tscd1		eor #%00011000
tretr0		eor #%00001100
twch0		eor #%00000110
tahed0		eor #%00000011
tscd0		eor #%00000001
		eor setup_features_exp
		sta setup_features_exp
		rts
                .if DRIVES=4
tretr3		eor #%11000000
twch3		eor #%01100000
tahed3		eor #%00110000
tscd3		eor #%00011000
tretr2		eor #%00001100
twch2		eor #%00000110
tahed2		eor #%00000011
tscd2		eor #%00000001
		eor setup_features2_exp
		sta setup_features2_exp
		rts
                .fi
tdirect		lda direct_write_exp
		eor #1
		sta direct_write_exp
		rts
;---------------------------------------
tboot		pha
		and #%11000000
		asl a
		rol a
		rol a
		tax
		pla
		and #%00111111
		ora pluss,x
		cpy #"-"
		bne ggdd
		and #%00111111
		ora minu,x
ggdd		sta setup_atribut_exp
		rts
;---------------------------------------
pluss		.byte %10000000
		.byte %10000000
minu		.byte %11000000
		.byte %00000000
		.byte %00000000
		.byte %10000000
;---------------------------------------
taccu		lda setup_accu
		jsr plusminus
                cmp #255
                bne *+4
                lda #6
                cmp #7
                bcc kek
                #lda0
kek		sta setup_accu
		rts
;---------------------------------------
plusminus	#adc1c			;0-15 +/- (color,etc.)
		cpy #"-"
		bne *+4
		sbc #2
		rts
;---------------------------------------
col		jsr plusminus
		and #15
		rts
;---------------------------------------
drv		jsr plusminus
		cmp #255
                bne *+4
                lda #30
                cmp #1
                bne *+4
                lda #8
		cmp #8
		bcc *+6
		cmp #31
		bcc kfv
		#lda0
kfv		rts
;---------------------------------------
tbg		#ldatax setup_paper_border_exp
		and #$f0
                sta tmp
                txa
tbgx		jsr col
		cmp setup_chars_exp
		beq tbgx
                ora tmp
		sta setup_paper_border_exp
		rts
;---------------------------------------
tfg		lda setup_paper_border_exp
		and #$0f
                sta tmp
                lda setup_chars_exp
tfgx		jsr col
                cmp tmp
		beq tfgx
		sta setup_chars_exp
		rts
;---------------------------------------
tdr11		inx
tdr10		inx
tdr9		inx
tdr8		inx
		.if DRIVES=4
tpcl		inx
tpm4	  	inx
		.fi
tpm2	  	inx
tmon2		inx
tbr		inx
		inx
tman13  	inx
tman12  	inx
tman11  	inx
tman10  	inx
tman9		inx
tman8   	lda setup_mancolors_exp,x
		pha
		and #$0f
                sta tmp
                pla
                jsr lsr4
		jsr col
                asl
                asl
                asl
                asl
                gcc cjc

		.if DRIVES=2
tpcl	  	inx
 		inx
		.fi
		.if DRIVES=4
thd3	  	inx
thd2	  	inx
		.fi
thd1		inx
thd0		inx
		.if DRIVES=4
tpm3	  	inx
		.fi
tpm1		inx
tmon1		inx
		inx
tman7		inx
tman6		inx
tman5		inx
tman4		inx
tman3		inx
tman2		inx
tman1   	lda setup_mancolors_exp,x
		pha
		and #$f0
                sta tmp
                pla
		jsr col
cjc             ora tmp
		sta setup_mancolors_exp,x
		rts
;---------------------------------------
		.comment
tlcd		.if DRIVES=4
		#ldatax setup_drivenumbers_exp+4
		.else
		#ldatax setup_drivenumbers_exp+2
                .fi
		and #$f0
                sta tmp
                txa
		and #$0f
		jsr plusminus
                cmp #255
                bne *+4
                lda #4
                cmp #5
                bcc bdb
                #lda0
bdb             ora tmp
		.if DRIVES=4
		sta setup_drivenumbers_exp+4
		.else
		sta setup_drivenumbers_exp+2
                .fi
		rts
                .endc
;---------------------------------------
tbootd		lda setup_boot_drive_exp
		jsr drv
		sta setup_boot_drive_exp
		rts
;---------------------------------------
		.if DRIVES=4
twin3		inx
twin2		inx
		.fi
twin1		inx
twin0		inx
		.if DRIVES=4
tlin3		inx
tlin2		inx
		.fi
tlin1		inx
tlin0		lda setup_linear_r_hdd0_exp,x
		jsr plusminus
		and #127
		sta setup_linear_r_hdd0_exp,x
		rts
;---------------------------------------
thr		#ldatax hour_exp
		jsr incbcd
		bcs *+4
		ldx #$23
		cpx #$24
		lda #$84		;writehour
		gne updatecx
;---------------------------------------
tmn		#ldatax min_exp
		jsr incbcd
		lda #$82		;writemin
tvsz		bcs *+4
		ldx #$59
		cpx #$60
updatecx	#jmps updatec
;---------------------------------------
tsec1		#ldatax sec_exp
		jsr incbcd
		lda #$80		;writesec
		bne tvsz
;---------------------------------------
tyr		#ldatax year_exp
		jsr incbcd
		bcs *+4
		ldx #$99
		cpx #$a0
		lda #$8c		;writeyear
		jsr updatec
		#sty_l_a year_exp
dfix2		lda date_exp
		#jmps dfix
;---------------------------------------
tdt		#ldatax date_exp
		jsr incbcd
		txa
dfix		bne *+5
		jsr monthmax
		sta date_exp
		jsr monthmax
		#ldx_l_a date_exp
		bcs *+4
		ldx #1
		lda #$86		;writedate
		gne updatec2

monthmax
		#ldx_l_a month_exp
		cpx #$02
		bne kqe
		lda year_exp
		jsr bcd2bin
		ldx #2
		and #3
		bne kqe
		ldx #10
kqe		lda monl-1,x
		cmp date_exp
		rts
;---------------------------------------
tmh		#ldatax month_exp
		jsr incbcd
		cpx #1
		bcs *+4
		ldx #$12
		cpx #$13
		bcc *+4
		ldx #1
		lda #$88		;writemonth
		jsr updatec2
		#sty_l_a month_exp
		#jmps dfix2
;---------------------------------------
updatec		bcc updatec2
		ldx #$00
updatec2	php
                pha
		#phx_a
                clc
                sei
		jsr cmos_writeprotect
		#ply_a
                pla
		jsr cmos_start
		tya
		jsr cmos_send_byte
		sta kill_port
                sec
		jsr cmos_writeprotect
                plp
;fall through
;---------------------------------------
incbcd		and #15
		cpy #"-"
		beq decbcd
                #adc1c
		cmp #10
		bcc gr
                #sbx_a 256-6
gr		inx
		sec
		rts
;---------------------------------------
decbcd		sbc #1
		bcs gr2
                .if SUPERCPU=1
                txa
                sbc #6-1
                tax
                .else
                lda #$ff
                sbx #6
                .fi
gr2		dex
		rts
;---------------------------------------
backlight2	#stz_a $d015
backlight	lda #7
		.byte SKIP2
highlight	lda #1
		pha
		ldx hol
		lda mpont,x
		asl a
		asl a
		asl a
		adc #24
		sta $d000
		#lda0
                bcc *+4
                lda #7
		sta $d010
		lda mpont+1,x
                tay
                and #$1f
		asl a
		sta tmp
		asl a
		asl a
		adc #50
		sta $d001
		sta $d003
		sta $d005
		sbc #49
		clc
		adc tmp
                asl
		sta tmp
		lda #$36
		rol a
		asl tmp
		rol a
		sta tmp+1
		tya
                jsr lsr4
                lsr
                tay
                lda fajt,y
		ldy mpont,x
		tax
		cpx #3
		#lda0
		bcc sw
		lda #%11110000
		cpx #4
		bcc sw
		lda #%11111111
sw		sta $0341
		sta $0344
		sta $0347
		sta $034a
		#lda0
		cpx #7
		bcc sw2
		lda #%11111111
sw2		sta $0342
		sta $0345
		sta $0348
		sta $034b
		lda $d000
		cpx #7
		bcc sw3
                adc #7
		cpx #8
		bcc sw3
		adc #7
		cpx #11
		bcc sw3
		adc #23
sw3		sta $d002
		cpx #14
		bcc sw4
		adc #23
		cpx #16
		bcc sw4
		adc #15
sw4		sta $d004

		pla
fv		sta (tmp),y
		iny
		dex
		bne fv
		rts
;---------------------------------------
printpg		sta tmp
		sty tmp+1

		lda #<$4a1
		ldy #>$4a1
		jsr settogg

		ldx #17
ig2		ldy #37
		lda #32
ig		sta (togg),y
		dey
		bpl ig
		lda togg+1
		pha
		eor #$d8^$04
		sta togg+1
		ldy #37
		lda #7
ig3		sta (togg),y
		dey
		bpl ig3
		pla
		sta togg+1
		lda togg
		clc
		adc #40
		sta togg
		bcc *+4
		inc togg+1
		dex
		bne ig2

		#sta_cfg_16

		clc
		ldy #1
		ldx #4
		jsr $fff0
printpgin       ldy #0
ve		lda (tmp),y
		beq thdho
                cmp #29
		bcs tkj
		cmp #17
                beq tkj
		cmp #13
                beq tkj
		cmp #5
                beq tkj
tk              sta $d3
                lda #29
tkj     	jsr newchrout_t
                cmp #13
                bne *+4
                inc $d3
xq		iny
		bne ve
		inc tmp+1
		bne ve
thdho		rts

;--------------------------------------
;Accu
;
accuconst      	.byte %10100000
		.byte %10101011
		.byte %10100111
		.byte %10101010
		.byte %10100110
		.byte %10101001
		.byte %10100101

		.enc screen
battlow		.text "Battery was low, check settings!"
                .enc none

showclock	= $07e8
hol		= $07e9
retry		= $07ea
m648		= $07eb
ism		= $07ec
datepuff	= $07ed
oldday		= $07ee
setup_accu    	= $07ef
setup_accu_old 	= $07f0
hdsettings      = $07f1
		.pend

;---------------------------------------
;Name of days and months
;
		.enc screen
dayc1		.text "SMTWTFSJFMAMJJASOND"
dayc2		.text "uouehraaeapauuuecoe"
dayc3		.text "nneduitnbrrynlgptvc"
                .enc none
